# The LearningOnline Network with CAPA 
# definition of tags that give a structure to a document
#
# $Id: bridgetask.pm,v 1.94 2005/12/21 19:13:05 albertel Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
###


package Apache::bridgetask; 

use strict;
use Apache::lonnet;
use Apache::File();
use Apache::lonmenu;
use Apache::lonlocal;
use Apache::lonxml;
use Apache::slotrequest();
use Time::HiRes qw( gettimeofday tv_interval );

BEGIN {
    &Apache::lonxml::register('Apache::bridgetask',('Task','IntroParagraph','Dimension','Instance','InstanceText','Criteria','ClosingParagraph'));
}

sub initialize_bridgetask {
    # id of current Dimension, 0 means that no dimension is current 
    # (inside <Task> only)
    $Apache::bridgetask::dimension='';
    # list of all Dimension ids seen
    @Apache::bridgetask::dimensionlist=();
    # mandatory attribute of all Dimensions seen
    %Apache::bridgetask::dimensionmandatory=();
    # list of all current Instance ids
    @Apache::bridgetask::instance=();
    # list of all Instance ids seen in this problem
    @Apache::bridgetask::instancelist=();
    # key of queud user data that we are currently grading
    $Apache::bridgetask::queue_key='';
}

sub proctor_check_auth {
    my ($slot_name,$slot,$type)=@_;
    my $user=$env{'form.proctorname'};
    my $domain=$env{'form.proctordomain'};
    
    my @allowed=split(",",$slot->{'proctor'});
    foreach my $possible (@allowed) {
	my ($puser,$pdom)=(split('@',$possible));
	if ($puser eq $user && $pdom eq $domain) {
	    my $authenticated=0;
	    if ( $slot->{'secret'} =~ /\S/ &&
		 $env{'form.proctorpassword'} eq $slot->{'secret'} ) {
		$authenticated=1;
	    } else {
		
		my $authhost=&Apache::lonnet::authenticate($puser,$env{'form.proctorpassword'},$pdom);
		if ($authhost ne 'no_host') {
		    $authenticated=1;
		}
	    }
	    if ($authenticated && $type eq 'Task') {
		# increment version
		my $version=
		    $Apache::lonhomework::history{'resource.0.version'};
		$version++;

		#clean out all current results
		foreach my $key (keys(%Apache::lonhomework::history)) {
		    if ($key=~/^resource\.0\./) {
			$Apache::lonhomework::results{$key}='';
		    }
		}
		
		#setup new version and who did it
		$Apache::lonhomework::results{'resource.0.version'}=$version;
		$Apache::lonhomework::results{"resource.$version.0.checkedin"}=
		    $user.'@'.$domain;
		$Apache::lonhomework::results{"resource.$version.0.checkedin.slot"}=
		    $slot_name;

		return 1;
	    } elsif ($authenticated && $type eq 'problem') {
		&Apache::lonxml::debug("authed $slot_name");
		$Apache::lonhomework::results{"resource.0.checkedin"}=
		    $user.'@'.$domain;
		$Apache::lonhomework::results{"resource.0.checkedin.slot"}=
		    $slot_name;
	    }
	}
    }
    return 0;
}

sub get_version {
    my ($version,$previous);
    if ($env{'form.previousversion'} && 
	$env{'form.previousversion'} ne 'current' &&
	defined($Apache::lonhomework::history{'resource.'.$env{'form.previousversion'}.'.0.status'})) {
	$version=$env{'form.previousversion'};
	$previous=1;
    } else {
	$version=$Apache::lonhomework::history{'resource.0.version'};
	$previous=0;
    }
    if (wantarray) {
	return ($version,$previous);
    }
    return $version;
}

sub add_previous_version_button {
    my ($status)=@_;
    my $result;
    if ($Apache::lonhomework::history{'resource.0.version'} eq '') {
	return '';
    }
    if ($Apache::lonhomework::history{'resource.0.version'} < 2 &&
	$status ne 'NEEDS_CHECKIN') {
	return '';
    }
    my $version=&get_version();
    if ($env{'form.previousversion'} ne '' &&
	$env{'form.previousversion'} eq $version) {
	$result.="<h3>".&mt("Showing previous version [_1]",$version).
	    "</h3>\n";
    }
    my @to_show;
    foreach my $test_version (1..$Apache::lonhomework::history{'resource.0.version'}) {
	if (defined($Apache::lonhomework::history{'resource.'.$test_version.'.0.status'})) {
	    push(@to_show,$test_version);
	}
    }
    my $list='<option>'.
	join("</option>\n<option>",@to_show).
	     "</option>\n";
    $list.='<option value="current">'.&mt('Current').'</option>';
    $result.='<form name="getprevious" method="POST" action="';
    my $uri=$env{'request.uri'};
    if ($env{'request.enc'}) { $uri=&Apache::lonenc::encrypted($uri); }
    $result.=$uri.'">'.
	&mt(' Show a previously done version: [_1]','<select onchange="this.form.submit()" name="previousversion">
<option>'.&mt('Pick one').'</option>
'.$list.'
</select>')."</form>";
    return $result;
}

sub add_grading_button {
    my (undef,$cid)=&Apache::lonxml::whichuser();
    my $cnum=$env{'course.'.$cid.'.num'};
    my $cdom=$env{'course.'.$cid.'.domain'};
    my %sections;
    my $numsections=&Apache::loncommon::get_sections($cdom,$cnum,\%sections);
    my $size=5;
    if (scalar(keys(%sections)) < 3) {
	$size=scalar(keys(%sections))+2;
    }
    my $sec_select = '<select multiple name="chosensections" size="'.$size.'">'."\n";
    $sec_select .= "<option value='all' selected='selected'>all</option>\n";
    foreach my $sec (sort {lc($a) cmp lc($b)} (keys(%sections))) {
	$sec_select .= "<option name=\"$sec\">$sec</option>\n";
    }
    $sec_select .= "<option value='none'>none</option></select>\n";
    
    my $result=' <input type="submit" name="gradeasubmission" value="'.
	&mt("Get a submission to grade").'" />';
    $result.='<input type="hidden" name="grade_target" value="webgrade" />';
    if (&Apache::lonnet::allowed('mgq',$env{'request.course.id'})) {
	my ($entries,$ready,$locks)=&get_queue_counts('gradingqueue');
	$result.='<p>Specify a section: '.$sec_select.'</p>';
	$result.='<p>'.&mt("Grading Queue has [_1] entries. [_2] of them are ready to be graded and [_3] of them are currently being graded",$entries,$ready,$locks);

	$result.=' <input type="submit" name="reviewagrading" value="'.
	    &mt("Select an entry from the grading queue").'" /> </p>'."\n";

	($entries,$ready,$locks)=&get_queue_counts('reviewqueue');
	$result.='<p>'.&mt("Review Queue has [_1] entries. [_2] of them are ready to be graded and [_3] of them are currently being graded",$entries,$ready,$locks);
	$result.=' <input type="submit" name="reviewasubmission" value="'.
	    &mt("Select an entry from the review queue").'" /> </p>'."\n";
	$result.=' <input type="submit" name="regradeasubmission" value="'.
	    &mt("Select a user to regrade.").'" /> </p>'."\n";
    }
    return $result;
}

sub add_request_another_attempt_button {
    my ($text)=@_;
    if (!$text) { $text="Request another attempt"; }
    my $result;
    my $symb=&Apache::lonnet::symbread();
    my ($slot_name,$slot)=&Apache::slotrequest::check_for_reservation($symb);
    my $action='get_reservation';
    if ($slot_name) {
	$text="Change reservation.";
	$action='change_reservation';
	my $description=&Apache::slotrequest::get_description($slot_name,
							      $slot);
	$result.=(<<STUFF);
<p> Will be next available: $description </p>
STUFF
    }
    
    if ($env{'request.enc'}) { $symb=&Apache::lonenc::encrypted($symb); }
    $symb=&Apache::lonnet::escape($symb);
    $result.='<form method="POST" action="/adm/slotrequest">'.
	'<input type="hidden" name="symb" value="'.$symb.'" />'.
	'<input type="hidden" name="command" value="'.$action.'" />'.
	'<input type="submit" name="requestattempt" value="'.
	&mt($text).'" />'.
	'</form>';
    return $result;
}

sub preserve_grade_info {
    my $result;
    # if we are viewing someone else preserve that info
    if (defined $env{'form.grade_symb'}) {
	foreach my $field ('symb','courseid','domain','username') {
	    $result .= '<input type="hidden" name="grade_'.$field.
		'" value="'.$env{"form.grade_$field"}.'" />'."\n";
	}
    }
    return $result;
}

sub style {
    return (<<STYLE);
<style type="text/css">
.fail, .pass, .neutral {
    position: relative;
    margin : 5px;
    margin-bottom :10px;
    padding : 4px;
    padding-left : 75px;
	border : thin solid;
    font-weight : bolder;
    font-size: smaller;
    font-family: Arial;
    background-color : rgb(255,240,225);
}

.fail h4, .pass h4 {
	position:absolute;
	left: -4px;
	top: -8px;
	padding:2px;
	margin:0;
	background-color : rgb(255,240,225);
	border : thin solid;
}

.fail {
	color: red;
}

.pass {
    color : green;
}

.neutral {
	color : blue;
}

.question {
	border : thin solid black;
	padding : 4px;
	margin-bottom : 1em;
}

.grade {
	font-size: x-large;
	font-family: Arial;
	position:absolute;
	left: 5px;
	top: -5px;
	width: 70px;
}

div#feedback h1 {
    color : inherit;
}
</style>
STYLE

}

sub show_task {
    my ($status,$previous)=@_;
    if (!$previous && (
		       ( $status eq 'CLOSED' ) ||
		       ( $status eq 'BANNED') ||
		       ( $status eq 'UNAVAILABLE') ||
		       ( $status eq 'NOT_IN_A_SLOT') ||
		       ( $status eq 'NEEDS_CHECKIN') ||
		       ( $status eq 'WAITING_FOR_GRADE') ||
		       ( $status eq 'INVALID_ACCESS') )) {
	return 0;
    }
    if ($env{'form.donescreen'}) { return 0; }
    return 1;
}

sub internal_location {
    my ($id)=@_;
    return '<!-- LONCAPA_INTERNAL_ADD_TASK_STATUS'.$id.' -->';
}

sub submission_time_stamp {
    my ($symb,$courseid,$udom,$uname)=&Apache::lonxml::whichuser();
    my $submissiontime;
    my $version=$Apache::lonhomework::history{'resource.0.version'};
    for (my $v=$Apache::lonhomework::history{'version'};$v>0;$v--) {
	if (defined($Apache::lonhomework::history{$v.':resource.'.$version.'.0.bridgetask.portfiles'})) {
	    $submissiontime=$Apache::lonhomework::history{$v.':timestamp'};
	}
    }
    my $result;
    if ($submissiontime) {
	my $slot_name=$Apache::lonhomework::history{'resource.'.$version.'.0.checkedin.slot'};
	my %slot=&Apache::lonnet::get_slot($slot_name);
	my $diff = $slot{'endtime'} - $submissiontime;
	my ($color,$when)=('#FF6666','after');
	if ($diff > 0) { ($color,$when)=('#336600','before'); }
	my $info;
	if ($diff%60) { $info=($diff%60).' seconds'; }
	$diff=int($diff/60);
	if ($diff%60) { $info=($diff%60).' minutes '.$info; }
	$diff=int($diff/60);
	if ($diff) {    $info=$diff.' hours '.$info; }
	$result='<p><font color="'.$color.'">'.
	    &mt('Student submitted [_1] [_2] the deadline. 
                 (Submission was at [_3], end of period was [_4].)',
		$info,$when,scalar(localtime($submissiontime)),
		scalar(localtime($slot{'endtime'}))).
		'</font></p>';
    }
    return $result;
}

sub webgrade_standard_info {
    my ($version)=&get_version();
    my (undef,undef,$udom,$uname) = &Apache::lonxml::whichuser();
    my $file_url = '/uploaded/'.$udom.'/'.$uname.'/portfolio/';
    my $file_list="<ul>\n";
    foreach my $partial_file (split(',',$Apache::lonhomework::history{"resource.$version.0.bridgetask.portfiles"})) {
	my $file=$file_url.$partial_file;
	$file=~s|/+|/|g;
	&Apache::lonnet::allowuploaded('/adm/bridgetask',$file);
	$file_list.='<li><nobr><a href="'.$file.'?rawmode=1" target="lonGRDs"><img src="'.
	    &Apache::loncommon::icon($file).'" border=0"> '.$file.
	    '</a></nobr></li>'."\n";
    }
    $file_list.="</ul>\n";

    my %lt=&Apache::lonlocal::texthash('done' => 'Done',
				       'stop' => 'Stop',
				       );
				       
    my $result=<<INFO;
<div class="handininfo">
  <input type="submit" name="next" value="$lt{'done'}" />
  <input type="submit" name="stop" value="$lt{'stop'}" />
  $file_list
</div>
INFO
    return $result;
}

sub start_Task {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;

    my ($status,$accessmsg,$slot);
    if ($target ne 'webgrade') {
	&Apache::structuretags::init_problem_globals('Task');
	&Apache::structuretags::initialize_storage();
	&Apache::lonhomework::showhash(%Apache::lonhomework::history);
	if ($env{'request.state'} eq 'construct') {
	    &Apache::structuretags::setup_rndseed($safeeval);
	}
    } 

    $Apache::lonhomework::parsing_a_task=1;
    #should get back a <html> or the neccesary stuff to start XML/MathML
    my ($result,$head_tag_start,$body_tag_start,$form_tag_start)=
	&Apache::structuretags::page_start($target,$token,$tagstack,$parstack,$parser,$safeeval);
    
    if ($target eq 'web' && $env{'request.state'} ne 'construct') {
	if ($Apache::lonhomework::modifygrades) {
	    $body_tag_start.='<form name="gradesubmission" method="POST" action="';
	    my $uri=$env{'request.uri'};
	    if ($env{'request.enc'}) { $uri=&Apache::lonenc::encrypted($uri); }
	    $body_tag_start.=$uri.'">'.&add_grading_button()."</form>";
	    my $symb=&Apache::lonnet::symbread();
	    if (&Apache::lonnet::allowed('mgq',$env{'request.course.id'})) {
		$body_tag_start.='<form method="POST" action="/adm/slotrequest">'.
		    '<input type="hidden" name="symb" value="'.$symb.'" />'.
		    '<input type="hidden" name="command" value="showslots" />'.
		    '<input type="submit" name="requestattempt" value="'.
		    &mt('Show Slot list').'" />'.
		    '</form>';
	    }
	}
    }
    if ($target eq 'web' && $env{'request.state'} eq 'construct') {
	$form_tag_start.=&Apache::structuretags::problem_web_to_edit_header($env{'form.rndseed'});
    }
    if ($target eq 'web' || ($target eq 'grade' && !$env{'form.webgrade'}) || $target eq 'answer' ||
	$target eq 'tex') {
	my ($version,$previous)=&get_version();
	($status,$accessmsg,my $slot_name,$slot) = 
	    &Apache::lonhomework::check_slot_access('0','Task');
	push(@Apache::inputtags::status,$status);
	$Apache::inputtags::slot_name=$slot_name;
	my $expression='$external::datestatus="'.$status.'";';
	$expression.='$external::gradestatus="'.$Apache::lonhomework::history{"resource.$version.0.solved"}.'";';
	&Apache::run::run($expression,$safeeval);
	&Apache::lonxml::debug("Got $status");
	$body_tag_start.=&add_previous_version_button($status);
	if (!&show_task($status,$previous)) {
	    my $bodytext=&Apache::lonxml::get_all_text("/task",$parser,$style);
	    if ( $target eq "web" ) {
		$result.= $head_tag_start.'</head>'.$body_tag_start;
		if ($env{'request.state'} eq 'construct') {
		    $result.=$form_tag_start;
		}
		my $msg;
		if ($status eq 'UNAVAILABLE') {
		    $msg.='<h1>'.&mt('Unable to determine if this resource is open due to network problems. Please try again later.').'</h1>';
		} elsif ($status eq 'NOT_IN_A_SLOT') {
		    $msg.='<h1>'.&mt('You are not currently signed up to work at this time and/or place.').'</h1>';
		    $msg.=&add_request_another_attempt_button("Sign up for time to work.");
		} elsif ($status eq 'NEEDS_CHECKIN') {
		    $msg.='<h1>'.&mt('You need the Proctor to validate you.').
			'</h1>'.&proctor_validation_screen($slot);
		} elsif ($status eq 'WAITING_FOR_GRADE') {
		    $msg.='<h1>'.&mt('Your submission is in the grading queue.').'</h1>';
		} elsif ($env{'form.donescreen'}) {
		    my $title=&Apache::lonnet::gettitle();
		    my @files=split(',',$Apache::lonhomework::history{'resource.'.$version.'.0.bridgetask.portfiles'});
		    my $files='<ul><li>'.join('</li><li>',@files).'</li></ul>';
		    $result.=<<DONESCREEN;
<h2>$title</h2>
<p> Files submitted: $files </p>
<p> You are now done with this Bridge Task </p>
<hr />
<p> <a href="/adm/logout">Logout</a> </p>
<p> <a href="/adm/roles">Change to a different course</a> </p>
DONESCREEN
		} elsif ($status ne 'NOT_YET_VIEWED') {
		    $msg.='<h1>'.&mt('Not open to be viewed').'</h1>';
		}
		if ($status eq 'CLOSED' || $status eq 'INVALID_ACCESS') {
		    $msg.='The problem '.$accessmsg;
		}
		$result.=$msg.'<br />';
	    } elsif ($target eq 'tex') {
		$result.='\begin{document}\noindent \vskip 1 mm  \begin{minipage}{\textwidth}\vskip 0 mm';
		if ($status eq 'UNAVAILABLE') {
		    $result.=&mt('Unable to determine if this resource is open due to network problems. Please try again later.').'\vskip 0 mm ';
		} else {
		    $result.=&mt('Problem is not open to be viewed. It')." $accessmsg \\vskip 0 mm ";
		}
	    } elsif ($target eq 'grade' && !$env{'form.webgrade'}) {
		if ($status eq 'NEEDS_CHECKIN') {
		    if(&proctor_check_auth($slot_name,$slot,'Task')
		       && defined($Apache::inputtags::slot_name)) {
			my $result=&add_to_queue('gradingqueue',
						 [$Apache::inputtags::slot_name]);
			&Apache::lonxml::debug("add_to_queue said $result");
		    }
		}
	    }
	} elsif ($target eq 'web') {
	    my $name= &Apache::structuretags::get_resource_name($parstack,$safeeval);
	    $result.="$head_tag_start<title>$name</title></head>
              $body_tag_start \n".&style();
	    
	    $result.=&preserve_grade_info();
	    $result.=&internal_location();
	    $result.=$form_tag_start.
		'<input type="hidden" name="submitted" value="yes" />';
	    &Apache::lonxml::startredirection();
	}
    } elsif ( ($target eq 'grade' && $env{'form.webgrade'}) ||
	      $target eq 'webgrade') {
	my $webgrade='yes';
	if ($target eq 'webgrade') {
	    $result.=$head_tag_start.$body_tag_start;
	    #$result.='<br />Review'.&show_queue('reviewqueue');
	    #$result.='<br />Grade'.&show_queue('gradingqueue');
	}
	# FIXME Blast! still need to reorg this, need to reshow the
        #       queue being reviewed once done with the grade pass...
        #       Hrrm, vaildation pass should perhaps say 'not_locked'
        #       perhaps do a search if there is a key that is mine and if
        #       there isn't reshow the queue....
	my ($todo,$status_code)=&get_key_todo($target);

	if ($todo) {
	    &setup_env_for_other_user($todo,$safeeval);
	    my ($symb,$uname,$udom)=&decode_queue_key($todo);
	    $result.="\n".'<table><tr><td>Found '.
		&Apache::lonnet::gettitle($symb).' for '.$uname.' at '.$udom.'</td></tr></table>';
	    $form_tag_start.=
		'<input type="hidden" name="gradingkey" value="'.
		&Apache::lonnet::escape($todo).'" />';
	    $Apache::bridgetask::queue_key=$todo;
	    &Apache::structuretags::initialize_storage();
	    &Apache::lonhomework::showhash(%Apache::lonhomework::history);
	    if ($target eq 'webgrade') {
		#$result.='<br />After -'.&show_queue($env{'form.queue'});
		$result.="\n".'<table width="100%" style="width:100%" border="1">';
		if ($status_code eq 'selected') {
		    $form_tag_start.=
			'<input type="hidden" name="queuemode" value="selected" />';
		}
	    }
	} else {
	    if ($target eq 'webgrade') {
		$result.="\n";
		my $back='<p><a href="/adm/flip?postdata=return:">'.
		    &mt('Return to resource').'</a></p>';
		if      ($status_code eq 'stop') {
		    $result.='<b>'.&mt("Stopped grading.").'</b>'.$back;
		} elsif ($status_code eq 'lock_failed') {
		    $result.='<b>'.&mt("Failed to lock the request record.")
			.'</b>'.$back;
		} elsif ($status_code eq 'unlock') {
		    $result.='<b>'.&mt("Unlocked the requested record.")
			.'</b>'.$back;
		    $result.=&show_queue($env{'form.queue'},1);
		} elsif ($status_code eq 'show_list') {
		    $result.=&show_queue($env{'form.queue'},1);
		} elsif ($status_code eq 'select_user') {
		    $result.=&select_user();
		} else {
		    $result.='<b>'.&mt("No user to be graded.").'</b>'.$back;
		}
	    }
	    $webgrade='no';
	    my $bodytext=&Apache::lonxml::get_all_text("/task",$parser,$style);
	}
	if ($target eq 'webgrade' && defined($env{'form.queue'})) {
	    if ($webgrade eq 'yes') {
		$result.=&submission_time_stamp();
	    }
	    $result.=$form_tag_start;
	    $result.='<input type="hidden" name="webgrade" value="'.
		$webgrade.'" />';
	    $result.='<input type="hidden" name="queue" value="'.
		$env{'form.queue'}.'" />';
	    if ($env{'form.regrade'}) {
		$result.='<input type="hidden" name="regrade" value="'.
		    $env{'form.regrade'}.'" />';
	    }
	    if ($env{'form.chosensections'}) {
		my @chosen_sections=
		    &Apache::loncommon::get_env_multiple('form.chosensections');
		foreach my $sec (@chosen_sections) {
		    $result.='<input type="hidden" name="chosensections" 
                               value="'.$sec.'" />';
		}
	    }
	    if ($webgrade eq 'yes') { $result.=&webgrade_standard_info(); }
	}
    } elsif ($target eq 'edit') {
	$result.=$head_tag_start."</head>".$body_tag_start.$form_tag_start.
	    &Apache::structuretags::problem_edit_header();
	$Apache::lonxml::warnings_error_header=
	    &mt("Editor Errors - these errors might not effect the running of the problem, but they will likely cause problems with further use of the Edit mode. Please use the EditXML mode to fix these errors.")."<br />";
	my $temp=&Apache::edit::insertlist($target,$token);
	$result.=$temp;
    } else {
	# page_start returned a starting result, delete it if we don't need it
	$result = '';
    }
    return $result;
}

sub get_key_todo {
    my ($target)=@_;
    my $todo;

    if (defined($env{'form.reviewasubmission'})) {
	&Apache::lonxml::debug("review a submission....");
	$env{'form.queue'}='reviewqueue';
	return (undef,'show_list');
    }

    if (defined($env{'form.reviewagrading'})) {
	&Apache::lonxml::debug("review a grading....");
	$env{'form.queue'}='gradingqueue';
	return (undef,'show_list');
    }

    if (defined($env{'form.regradeasubmission'})) {
	&Apache::lonxml::debug("regrade a grading....");
	$env{'form.queue'}='none';
	return (undef,'select_user');
    }

    my $queue=$env{'form.queue'};

    if (!defined($queue)) {
	$env{'form.queue'}=$queue='gradingqueue';
    }

    my $gradingkey=&Apache::lonnet::unescape($env{'form.gradingkey'});

    if ($env{'form.queue'} eq 'none') {
	if (defined($env{'form.gradingkey'})) {
	    if ($target eq 'webgrade') {
		if ($env{'form.stop'}) {
		    return (undef,'stop');
		} elsif ($env{'form.next'}) {
		    return (undef,'select_user');
		}
	    }
	    return ($gradingkey,'selected');
	} else {
	    return (undef,'select_user');
	}
    }
    if (defined($env{'form.queue'}) && defined($env{'form.gradingkey'})
	&& !defined($env{'form.gradingaction'}) 
	&& $env{'form.queuemode'} eq 'selected') {
	
	my $who=&queue_key_locked($queue,$gradingkey);
	my $me=$env{'user.name'}.'@'.$env{'user.domain'};
	if ($who eq $me) {
	    &Apache::lonxml::debug("Found a key was given to me");
	    return ($gradingkey,'selected');
	} else {
	    return (undef,'show_list');
	}

    }

    if ($target eq 'webgrade' && $env{'form.queuemode'} eq 'selected') {
	if ($env{'form.gradingaction'} eq 'resume') {
	    delete($env{'form.gradingaction'});
	    &Apache::lonxml::debug("Resuming a key");
	    return ($gradingkey);
	} elsif ($env{'form.gradingaction'} eq 'unlock') {
	    &Apache::lonxml::debug("Unlocking a key ".
				     &check_queue_unlock($queue,$gradingkey,1));
	    return (undef,'unlock');
	} elsif ($env{'form.gradingaction'} eq 'select') {
	    &Apache::lonxml::debug("Locking a key");
	    if (&lock_key($queue,$gradingkey)) {
		&Apache::lonxml::debug("Success $queue");
		return ($gradingkey);
	    }
	    &Apache::lonxml::debug("Failed $queue");
	    return (undef,'lock_failed');
	}
    }

    if ($env{'form.queuemode'} ne 'selected') {
	# don't get something new from the queue if they hit the stop button
    	if (!($env{'form.stop'} && $target eq 'webgrade') 
	    && !$env{'form.gradingaction'}) {
	    &Apache::lonxml::debug("Getting anew $queue");
	    return (&get_from_queue($queue));
	} else {
	    return (undef,'stop');
	}
    }
    return (undef,undef)
}

sub minimize_storage {
    foreach my $key (keys(%Apache::lonhomework::results)) {
	if ($key =~ /regrader$/) { next; }
	if ($Apache::lonhomework::results{$key} eq
	    $Apache::lonhomework::history{$key}) {
	    delete($Apache::lonhomework::results{$key});
	}
    }
}

sub end_Task {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
    my $result='';
    my $status=$Apache::inputtags::status['-1'];
    my ($version,$previous)=&get_version();
    if ($target eq 'grade' || $target eq 'web' || $target eq 'answer' ||
	$target eq 'tex') {
	if ($target eq 'web' || $target eq 'answer' || $target eq 'tex') {
	    if ($target eq 'web') {
		if (&show_task($status,$previous)) {
		    $result.=&Apache::lonxml::endredirection();
		}
		if ($status eq 'CAN_ANSWER' && !$previous && 
		    !$env{'form.donescreen'}) {
		    $result.="\n".'<table border="1">'.
			&Apache::inputtags::file_selector("$version.0",
							  "bridgetask","*",
							  'portfolioonly',
							  '
<h2>'.&mt('Submit Portfolio Files for Grading').'</h2>
<p>'.&mt('Indicate the files from your portfolio to be evaluated in grading this task.').'</p>').
							  "</table>";
		}
		if (!$previous && $status ne 'SHOW_ANSWER' &&
		    &show_task($status,$previous)) {
		    $result.=&Apache::inputtags::gradestatus('0');
		    $result.='</form>';
		    $result.=<<DONEBUTTON;
<form method="POST">
   <input type="hidden" name="donescreen" value="1" />
   <input type="submit" value="Done" />
</form>
DONEBUTTON
                }
		if (&show_task($status,$previous) &&
		    $Apache::lonhomework::history{"resource.$version.0.status"} =~ /^(pass|fail)$/) {
		    my $bt_status=$Apache::lonhomework::history{"resource.$version.0.status"};
		    my $title=&Apache::lonnet::gettitle();

		    my $slot_name=
			$Apache::lonhomework::history{"resource.$version.0.checkedin.slot"};
		    my %slot=&Apache::lonnet::get_slot($slot_name);
		    my $start_time=
			&Apache::lonlocal::locallocaltime($slot{'starttime'});

		    my $status = "\n<div class='$bt_status'>\n";
		    
		    if ($bt_status eq 'pass')  {
			$status.='<h2>You passed the '.$title.' given on '.
			    $start_time.'</h2>';
		    }
		    if ($bt_status eq 'fail')  {
			$status.='<h2>You did not pass the '.$title.' given on '.
			    $start_time.'</h2>';
			if (!$previous) {
			    $status.=&add_request_another_attempt_button();
			}
		    }
		    my $man_count=0;
		    my $opt_count=0;
		    my $opt_passed=0;
		    foreach my $dim_id (@Apache::bridgetask::dimensionlist) {
			if ($Apache::bridgetask::dimensionmandatory{$dim_id}
			    eq 'N') {
			    $opt_count++;
			    if ($Apache::lonhomework::history{"resource.$version.0.$dim_id.status"} eq 'pass') {
				$opt_passed++;
			    }
			} else {
			    $man_count++;
			}
		    }
		    my $opt_req=&Apache::lonxml::get_param('OptionalRequired',
							 $parstack,$safeeval);
		    if ($opt_req !~ /\S/) { $opt_req='0'; }
		    $status.="\n<p>".&mt('You needed to pass all of the [_1]  mandatory components and [_2] of the [_3] optional components, of which you were required to pass [_4] on the bridge task.',$man_count,$opt_passed,$opt_count,$opt_req)."</p></div>\n";

		    my $internal_location=&internal_location();
		    $result=~s/\Q$internal_location\E/$status/;
		}
	    } 
	    if ($target eq 'web' || $target eq 'webgrade') {
		$result.=&Apache::lonxml::xmlend().'</html>';
	    }
	}
	if ($target eq 'grade' && !$env{'form.webgrade'} && !$previous) {
	    my $award='SUBMITTED';
	    &Apache::essayresponse::file_submission("$version.0",'bridgetask',
						    'portfiles',\$award);
	    if ($award eq 'SUBMITTED' &&
		$Apache::lonhomework::results{"resource.$version.0.bridgetask.portfiles"}) {
		$Apache::lonhomework::results{"resource.0.tries"}=
		    $Apache::lonhomework::results{"resource.$version.0.tries"}=
		    1+$Apache::lonhomework::history{"resource.$version.0.tries"};

		$Apache::lonhomework::results{"resource.0.award"}=
		    $Apache::lonhomework::results{"resource.$version.0.award"}=
		    $award;
		$Apache::lonhomework::results{"resource.0.submission"}=
		    $Apache::lonhomework::results{"resource.$version.0.submission"}='';
	    } else {
		delete($Apache::lonhomework::results{"resource.$version.0.bridgetask.portfiles"});
		$award = '';
	    }
	    &Apache::lonhomework::showhash(%Apache::lonhomework::results);
	    &Apache::structuretags::finalize_storage();
	    if ($award eq 'SUBMITTED'
		&& defined($Apache::inputtags::slot_name)) {
		&add_to_queue('gradingqueue',[$Apache::inputtags::slot_name]);
	    }
	} elsif ($Apache::lonhomework::results{'INTERNAL_store'}) {
	    &Apache::structuretags::finalize_storage();
	}
	if ($target eq 'grade' && $env{'form.webgrade'} eq 'yes') {
	    my $optional_required=
		&Apache::lonxml::get_param('OptionalRequired',$parstack,
					   $safeeval);
	    my $optional_passed=0;
	    my $mandatory_failed=0;
	    my $ungraded=0;
	    my $review=0;   
	    &Apache::lonhomework::showhash(%Apache::lonhomework::results);
	    foreach my $dim_id (@Apache::bridgetask::dimensionlist) {
		my $status=
		    $Apache::lonhomework::results{"resource.$version.0.$dim_id.status"};
		my $mandatory=
		    ($Apache::bridgetask::dimensionmandatory{$dim_id} ne 'N');
		if ($status eq 'pass') {
		    if (!$mandatory) { $optional_passed++; }
		} elsif ($status eq 'fail') {
		    if ($mandatory) { $mandatory_failed++; }
		} elsif ($status eq 'ungraded') {
		    $ungraded++;
		} elsif ($status eq 'review') {
		    $review++;
		} else {
		    $ungraded++;
		}
	    }
	    if ($optional_passed < $optional_required) {
		$mandatory_failed++;
	    }
	    &Apache::lonxml::debug("all dim ".join(':',@Apache::bridgetask::dimensionlist)."results -> m_f $mandatory_failed o_p $optional_passed u $ungraded r $review");
	    $Apache::lonhomework::results{'resource.0.regrader'}=
		$env{'user.name'}.'@'.$env{'user.domain'};
	    if ($review) {
		$Apache::lonhomework::results{"resource.$version.0.status"}='review';
		if ($env{'form.queue'} eq 'reviewqueue') {
		    &check_queue_unlock($env{'form.queue'});
		    &Apache::lonxml::debug(" still needs review not changing status.");
		} else {
		    &move_between_queues($env{'form.queue'},'reviewqueue');
		}
	    } elsif ($ungraded) {
		$Apache::lonhomework::results{"resource.$version.0.status"}='ungraded';
		if ($env{'form.queue'} eq 'reviewqueue' ||
		    $env{'form.queue'} eq 'none' ) {
		    &Apache::lonxml::debug("moving back.");
		    &move_between_queues($env{'form.queue'},'gradingqueue');
		} else {
		    &check_queue_unlock($env{'form.queue'});
		}
	    } elsif ($mandatory_failed) {
		$Apache::lonhomework::results{"resource.$version.0.status"}='fail';
		$Apache::lonhomework::results{"resource.$version.0.solved"}='incorrect_by_override';
		$Apache::lonhomework::results{"resource.$version.0.award"}='INCORRECT';
		$Apache::lonhomework::results{"resource.$version.0.awarded"}='0';
		&remove_from_queue($env{'form.queue'}); 

		my ($symb,$courseid,$udom,$uname)=&Apache::lonxml::whichuser();
		
		if ($env{'form.regrade'} ne 'yes') {
		    $Apache::lonhomework::results{"resource.$version.0.bridgetask.portfiles"}=
			$Apache::lonhomework::history{"resource.$version.0.bridgetask.portfiles"};
		    &Apache::grades::version_portfiles(
						       \%Apache::lonhomework::results,
						       ["$version.0.bridgetask"],$courseid,
						       $symb,$udom,$uname,
						       ["$version.0.bridgetask"]);
		}
	    } else {
		$Apache::lonhomework::results{"resource.$version.0.status"}='pass';
		$Apache::lonhomework::results{"resource.$version.0.solved"}='correct_by_override';
		$Apache::lonhomework::results{"resource.$version.0.award"}='EXACT_ANS';
		$Apache::lonhomework::results{"resource.$version.0.awarded"}='1';
		&remove_from_queue($env{'form.queue'});

		my ($symb,$courseid,$udom,$uname)=&Apache::lonxml::whichuser();
		if ($env{'form.regrade'} ne 'yes') {
		    $Apache::lonhomework::results{"resource.$version.0.bridgetask.portfiles"}=
			$Apache::lonhomework::history{"resource.$version.0.bridgetask.portfiles"};
		    &Apache::grades::version_portfiles(
						       \%Apache::lonhomework::results,
						       ["$version.0.bridgetask"],$courseid,
						       $symb,$udom,$uname,
						       ["$version.0.bridgetask"]);
		}
	    }
	    $Apache::lonhomework::results{"resource.0.status"}=
		$Apache::lonhomework::results{"resource.$version.0.status"};
	    if (defined($Apache::lonhomework::results{"resource.$version.0.awarded"})) {
		$Apache::lonhomework::results{"resource.0.award"}=
		    $Apache::lonhomework::results{"resource.$version.0.award"};
		$Apache::lonhomework::results{"resource.0.awarded"}=
		    $Apache::lonhomework::results{"resource.$version.0.awarded"};
		$Apache::lonhomework::results{"resource.0.solved"}=
		    $Apache::lonhomework::results{"resource.$version.0.solved"};
	    }
	    &minimize_storage();
	    &Apache::structuretags::finalize_storage();
	}
    } elsif ($target eq 'webgrade') {
	$result.="</table>\n<hr />";
	#$result.='<input type="submit" name="next" value="'.
	#    &mt('Save &amp; Next').'" /> ';
	#$result.='<input type="submit" name="end" value="'.
	#    &mt('Save &amp; Stop Grading').'" /> ';
	#$result.='<input type="submit" name="throwaway" value="'.
	#    &mt('Throw Away &amp; Stop Grading').'" /> ';
	#$result.='<input type="submit" name="save" value="'.
	#    &mt('Save Partial Grade and Continue Grading').'" /> ';
	$result.='</form>'.&Apache::loncommon::endbodytag().'</html>';
    } elsif ($target eq 'meta') {
	$result.=&Apache::response::meta_package_write('Task');
        $result.=&Apache::response::meta_stores_write('solved','string',
						      'Problem Status');
	$result.=&Apache::response::meta_stores_write('tries','int_zeropos',
						      'Number of Attempts');
	$result.=&Apache::response::meta_stores_write('awarded','float',
						      'Partial Credit Factor');
	$result.=&Apache::response::meta_stores_write('status','string',
						      'Bridge Task Status');
    }
    undef($Apache::lonhomework::parsing_a_task);
    return $result;
}

sub move_between_queues {
    my ($src_queue,$dest_queue)=@_;
    my $cur_data;
    if ($src_queue ne 'none') {
	$cur_data=&get_queue_data($src_queue);
	if (!$cur_data) { return 'not_exist'; }
    } else {
	$cur_data = ['none'];
    }
    my $result=&add_to_queue($dest_queue,[$cur_data->[0]]);
    if ($result ne 'ok') {
	return $result;
    }
    &check_queue_unlock($src_queue);
    return &remove_from_queue($src_queue);
}

sub check_queue_unlock {
    my ($queue,$key,$allow_not_me)=@_;
    if ($queue eq 'none') { return 'ok'; }
    my ($symb,$cid,$udom,$uname)=&Apache::lonxml::whichuser();
    if (!defined($key)) {
	$key="$symb\0queue\0$uname\@$udom";
    }
    my $cnum=$env{'course.'.$cid.'.num'};
    my $cdom=$env{'course.'.$cid.'.domain'};
    my $me=$env{'user.name'}.'@'.$env{'user.domain'};
    my $who=&queue_key_locked($queue,$key,$cdom,$cnum);
    if  ($who eq $me) {
	return &Apache::lonnet::del($queue,["$key\0locked"],$cdom,$cnum);
    } elsif ($allow_not_me) {
	&Apache::lonxml::debug("unlocking $who by $me");
	return &Apache::lonnet::del($queue,["$key\0locked"],$cdom,$cnum);
    }
    return 'not_owner';
}

sub in_queue {
    my ($queue,$symb,$cdom,$cnum,$udom,$uname)=@_;
    if ($queue eq 'none') { return 0; }
    if (!defined($symb) || !defined($cdom) || !defined($cnum)
	|| !defined($udom) || !defined($uname)) {
	($symb,my $cid,$udom,$uname)=&Apache::lonxml::whichuser();
	$cnum=$env{'course.'.$cid.'.num'};
	$cdom=$env{'course.'.$cid.'.domain'};
    }

    my $key=&encode_queue_key($symb,$udom,$uname);
    my %results = &Apache::lonnet::get($queue,[$key],$cdom,$cnum);

    if (defined($results{$key})) {
	return 1;
    }
    return 0;
}

sub remove_from_queue {
    my ($queue,$symb,$cdom,$cnum,$udom,$uname)=@_;
    if ($queue eq 'none') { return 'ok'; }
    if (!defined($symb) || !defined($cdom) || !defined($cnum)
	|| !defined($udom) || !defined($uname)) {
	($symb,my $cid,$udom,$uname)=&Apache::lonxml::whichuser();
	$cnum=$env{'course.'.$cid.'.num'};
	$cdom=$env{'course.'.$cid.'.domain'};
    }
    if (!&in_queue($queue,$symb,$cdom,$cnum,$udom,$uname)) {
	return 'ok';
    }
    my $key=&encode_queue_key($symb,$udom,$uname);
    my @keys=($key,"$key\0locked");
    return &Apache::lonnet::del($queue,\@keys,$cdom,$cnum);
}

sub setup_env_for_other_user {
    my ($queue_key,$safeeval)=@_;
    my ($symb,$uname,$udom)=&decode_queue_key($queue_key);
    &Apache::lonxml::debug("setup_env for $queue_key");
    $env{'form.grade_symb'}=$symb;
    $env{'form.grade_domain'}=$udom;
    $env{'form.grade_username'}=$uname;
    $env{'form.grade_courseid'}=$env{'request.course.id'};
    &Apache::lonxml::initialize_rndseed($safeeval);
}

sub get_queue_data {
    my ($queue)=@_;
    my ($symb,$cid,$udom,$uname)=&Apache::lonxml::whichuser();
    my $cnum=$env{'course.'.$cid.'.num'};
    my $cdom=$env{'course.'.$cid.'.domain'};
    my $todo="$symb\0queue\0$uname\@$udom";
    my ($key,$value)=&Apache::lonnet::get($queue,[$todo],$cdom,$cnum);
    if ($key eq $todo && ref($value)) {
	return $value;
    }
    return undef;
}


sub check_queue_for_key {
    my ($cdom,$cnum,$queue,$todo)=@_;

    my %results=
	&Apache::lonnet::get($queue,[$todo,"$todo\0locked"],$cdom,$cnum);
    
    if (exists($results{$todo}) && ref($results{$todo})) {
	if (defined($results{"$todo\0locked"})) {
	    return 'locked';
	}
	my $slot;
	if (ref($results{$todo}) eq 'ARRAY')  {
	    $slot = $results{$todo}[0];
	} elsif (ref($results{$todo}) eq 'HASH')  {
	    $slot = $results{$todo}{'slot'};
	}
	if (defined($slot)) {
	    my %slot_data=&Apache::lonnet::get_slot($slot);
	    if ($slot_data{'endtime'} > time) { 
		return 'in_progress';
	    }
	}
	return 'enqueued';
    }
    return undef;
}

sub add_to_queue {
    my ($queue,$user_data)=@_;
    if ($queue eq 'none') { return 'ok'; }
    my ($symb,$cid,$udom,$uname)=&Apache::lonxml::whichuser();
    if (!$cid || $env{'request.state'} eq 'construct') {
	return 'no_queue';
    }
    my $cnum=$env{'course.'.$cid.'.num'};
    my $cdom=$env{'course.'.$cid.'.domain'};
    my %data;
    $data{"$symb\0queue\0$uname\@$udom"}=$user_data;
    return &Apache::lonnet::cput($queue,\%data,$cdom,$cnum);
}

sub show_queue {
    my ($queue,$with_selects)=@_;
    my $result;
    my ($symb,$cid,$udom,$uname)=&Apache::lonxml::whichuser();
    my $cnum=$env{'course.'.$cid.'.num'};
    my $cdom=$env{'course.'.$cid.'.domain'};

    my @chosen_sections=
	&Apache::loncommon::get_env_multiple('form.chosensections');
    &Apache::grades::init_perm();
    my ($classlist,$section,$fullname)=&Apache::grades::getclasslist(\@chosen_sections,);
    &Apache::grades::reset_perm();
    if (!(grep(/^all$/,@chosen_sections))) {
	$result.='<p> Showing only sections <tt>'.join(', ',@chosen_sections).
	    '</tt>.</p> '."\n";
    }

    my $regexp="^$symb\0";
    my %queue=&Apache::lonnet::dump($queue,$cdom,$cnum,$regexp);
    my ($tmp)=%queue;
    if ($tmp=~/^error: 2 /) {
	return "\n<h3>Current Queue - $queue</h3><table border='1'><tr><td>Empty</td></tr></table>";
    }
    $result.="\n<h3>Current Queue - $symb $queue </h3><table border='1'><tr>";
    if ($with_selects) { $result.="<th></th>"; }
    $result.="<th>resource</th><th>user</th><th>type</th><th>data</th></tr>";
    foreach my $key (sort(keys(%queue))) {
	my ($symb,$uname,$udom) = &decode_queue_key($key);
	if (!defined($classlist->{$uname.':'.$udom})) { next; }
	if ($key=~/locked$/ && !$with_selects) {
	    my $title=&Apache::lonnet::gettitle($symb);
	    $result.="<tr><td>$title</td><td>$uname</td>";
	    $result.='<td>lock</td><td>'.$queue{$key}.'</td></tr>';
	} elsif ($key=~/timestamp$/ && !$with_selects) {
	    my ($symb,undef) = split("\0",$key);
	    my $title=&Apache::lonnet::gettitle($symb);
	    $result.="<tr><td>$title</td><td></td>";
	    $result.='<td>last queue modification time</td><td>'.
		&Apache::lonlocal::locallocaltime($queue{$key})."</td></tr>";
	} elsif ($key!~/(timestamp|locked)$/) {
	    my $title=&Apache::lonnet::gettitle($symb);
	    $result.="<tr>";
	    my $slot=$queue{$key}->[0];
	    my %slot_data=&Apache::lonnet::get_slot($slot);
	    if ($with_selects) {
		my $ekey=&Apache::lonnet::escape($key);
		my ($action,$description)=('select',&mt('Select'));
		if (exists($queue{"$key\0locked"})) {
		    my $me=$env{'user.name'}.'@'.$env{'user.domain'};
		    if ($me eq $queue{"$key\0locked"}) {
			($action,$description)=('resume',&mt('Resume'));
		    } else {
			($action,$description)=('unlock',&mt('Unlock'));
		    }
		}
		my $seclist;
		foreach my $sec (@chosen_sections) {
		    $seclist.='<input type="hidden" name="chosensections" 
                               value="'.$sec.'" />';
		}
		if (time > $slot_data{'endtime'}) {
		    $result.=(<<FORM);
<td>
<form method="POST">
 <input type="hidden" name="gradingkey" value="$ekey" />
 <input type="hidden" name="queue" value="$queue" />
 <input type="hidden" name="gradingaction" value="$action" />
 <input type="hidden" name="webgrade" value="no" />
 <input type="hidden" name="queuemode" value="selected" />
 <input type="submit" name="submit" value="$description" />
 $seclist
</form>
</td>
FORM

                } else {
		    $result.='<td>'.&mt("In Progress").'</td>'
		}
	    }
	    $result.="<td>$title</td><td>$uname</td>";
	    $result.='<td>queue entry</td><td>Slot: '.$slot.' End time: '.
		&Apache::lonlocal::locallocaltime($slot_data{'endtime'}).
		"</td></tr>";
	}
    }
    $result.="</table><hr />\n";
    return $result;
}

sub get_queue_counts {
    my ($queue)=@_;
    my $result;
    my ($symb,$cid,$udom,$uname)=&Apache::lonxml::whichuser();
    my $cnum=$env{'course.'.$cid.'.num'};
    my $cdom=$env{'course.'.$cid.'.domain'};
    my $regexp="^$symb\0";
    my %queue=&Apache::lonnet::dump($queue,$cdom,$cnum,$regexp);
    my ($tmp)=%queue;
    if ($tmp=~/^error: 2 /) {
	return (0,0,0);
    }
    my ($entries,$ready_to_grade,$locks)=(0,0,0);
    foreach my $key (sort(keys(%queue))) {
	if ($key=~/locked$/) {
	    $locks++;
	} elsif ($key=~/timestamp$/) {
	    #ignore
	} elsif ($key!~/(timestamp|locked)$/) {
	    my ($symb,$uname,$udom) = &decode_queue_key($key);
	    $entries++;
	    my $slot=$queue{$key}->[0];
	    my %slot_data=&Apache::lonnet::get_slot($slot);
	    if (time > $slot_data{'endtime'}) {
		$ready_to_grade++;
	    }
	}
    }
    return ($entries,$ready_to_grade,$locks);
}

sub encode_queue_key {
    my ($symb,$udom,$uname)=@_;
    return "$symb\0queue\0$uname\@$udom";
}

sub decode_queue_key {
    my ($key)=@_;
    my ($symb,undef,$user) = split("\0",$key);
    my ($uname,$udom) = split('@',$user);
    return ($symb,$uname,$udom);
}

sub queue_key_locked {
    my ($queue,$key,$cdom,$cnum)=@_;
    if (!defined($cdom) || !defined($cnum)) {
	my (undef,$cid)=&Apache::lonxml::whichuser();
	$cnum=$env{'course.'.$cid.'.num'};
	$cdom=$env{'course.'.$cid.'.domain'};
    }
    my ($key_locked,$value)=
	&Apache::lonnet::get($queue,["$key\0locked"],$cdom,$cnum);
    if ($key_locked eq "$key\0locked") {
	return $value;
    }
    return undef;
}

sub pick_from_queue_data {
    my ($queue,$check_section,$queuedata,$cdom,$cnum)=@_;
    foreach my $key (keys(%$queuedata)) {
	if ($key =~ /\0locked$/) { next; }
	if ($key =~ /\0timestamp$/) { next; }
	my ($symb,$uname,$udom)=&decode_queue_key($key);
	if ($check_section) {
	    my $section=&Apache::lonnet::getsection($uname,$udom);
	    if ($section eq $check_section) {
		&Apache::lonxml::debug("my sec");
		next;
	    }
	}
	my $slot=$queuedata->{$key}[0];
	my %slot_data=&Apache::lonnet::get_slot($slot);
	if ($slot_data{'endtime'} > time) { 
	    &Apache::lonxml::debug("not time");
	    next;
	}
	if (&queue_key_locked($queue,$key,$cdom,$cnum)) {
	    &Apache::lonxml::debug("someone already has um.");
	    next;
	}
	return $key;
    }
    return undef;
}

sub find_mid_grade {
    my ($queue,$symb,$cdom,$cnum)=@_;
    my $todo=&Apache::lonnet::unescape($env{'form.gradingkey'});
    my $me=$env{'user.name'}.'@'.$env{'user.domain'};
    if ($todo) {
	my $who=&queue_key_locked($queue,$todo,$cdom,$cnum);
	if ($who eq $me) { return $todo; }
    }
    my $regexp="^$symb\0.*\0locked\$";
    my %locks=&Apache::lonnet::dump($queue,$cdom,$cnum,$regexp);
    foreach my $key (keys(%locks)) {
	my $who=$locks{$key};
	if ($who eq $me) {
	    $todo=$key;
	    $todo=~s/\0locked$//;
	    return $todo;
	}
    }
    return undef;
}

sub lock_key {
    my ($queue,$todo)=@_;
    my $me=$env{'user.name'}.'@'.$env{'user.domain'};
    my (undef,$cid)=&Apache::lonxml::whichuser();
    my $cnum=$env{'course.'.$cid.'.num'};
    my $cdom=$env{'course.'.$cid.'.domain'};
    my $success=&Apache::lonnet::newput($queue,{"$todo\0locked"=> $me},
					$cdom,$cnum);
    &Apache::lonxml::debug("success $success $todo");
    if ($success eq 'ok') {
	return 1;
    }
    return 0;
}

sub get_queue_symb_status {
    my ($queue,$symb,$cdom,$cnum) = @_;
    if (!defined($cdom) || !defined($cnum)) {
	my (undef,$cid)=&Apache::lonxml::whichuser();
	$cnum=$env{'course.'.$cid.'.num'};
	$cdom=$env{'course.'.$cid.'.domain'};
    }
    my $regexp="^$symb\0";
    my %queue=&Apache::lonnet::dump($queue,$cdom,$cnum,$regexp);
    my ($tmp)=%queue;
    if ($tmp=~/^error: 2 /) { return; }
    my @users;
    foreach my $key (sort(keys(%queue))) {
	next if ($key=~/locked$/);
	next if ($key=~/timestamp$/);
	my ($symb,$uname,$udom) = &decode_queue_key($key);
	push(@users,"$uname:$udom");
    }
    return @users;
}

sub get_from_queue {
    my ($queue)=@_;
    my $result;
    my ($symb,$cid,$udom,$uname)=&Apache::lonxml::whichuser();
    my $cnum=$env{'course.'.$cid.'.num'};
    my $cdom=$env{'course.'.$cid.'.domain'};
    my $todo=&find_mid_grade($queue,$symb,$cdom,$cnum);
    &Apache::lonxml::debug("found ".join(':',&decode_queue_key($todo)));
    if ($todo) { return $todo; }
    while (1) {
	my $starttime=time;
	&Apache::lonnet::cput($queue,{"$symb\0timestamp"=>$starttime},
			      $cdom,$cnum);
	&Apache::lonxml::debug("$starttime");
	my $regexp="^$symb\0queue\0";
	my %queue=&Apache::lonnet::dump($queue,$cdom,$cnum,$regexp);
	#make a pass looking for a user _not_ in my section
	if ($env{'request.course.sec'}) {
	    &Apache::lonxml::debug("sce");
	    $todo=&pick_from_queue_data($queue,$env{'request.course.sec'},
					\%queue,$cdom,$cnum);
	    &Apache::lonxml::debug("sce $todo");
	}
	# no one _not_ in our section so look for any user that is
	# ready for grading
	if (!$todo) {
	    &Apache::lonxml::debug("no sce");
	    $todo=&pick_from_queue_data($queue,$env{'request.course.sec'},
					\%queue,$cdom,$cnum);
	    &Apache::lonxml::debug("no sce $todo");
	}
	# no user to grade 
	if (!$todo) { last; }
	&Apache::lonxml::debug("got $todo");
	# otherwise found someone so lets try to lock them
	# unless someone else already picked them
	if (!&lock_key($queue,$todo)) { next; }
	my (undef,$endtime)=
	    &Apache::lonnet::get($queue,["$symb\0timestamp"],
				 $cdom,$cnum);
	&Apache::lonxml::debug("emd  $endtime");
	# someone else already modified the queue, 
	# perhaps our picked user wass already fully graded between
	# when we picked him and when we locked his record? so lets
	# double check.
	if ($endtime != $starttime) {
	    my ($key,$value)=
		&Apache::lonnet::get($queue,["$todo"],
				     $cdom,$cnum);
	    &Apache::lonxml::debug("check  $key .. $value");
	    if ($key eq $todo && ref($value)) {
	    } else {
		&Apache::lonnet::del($queue,["$todo\0locked"],
				     $cdom,$cnum);
		&Apache::lonxml::debug("del");
		next;
	    }
	}
	&Apache::lonxml::debug("last $todo");
	last;
    }
    return $todo;
}

sub select_user {
    my ($symb,$cid)=&Apache::lonxml::whichuser();

    my @chosen_sections=
	&Apache::loncommon::get_env_multiple('form.chosensections');
    &Apache::grades::init_perm();
    my ($classlist,$section,$fullname)=&Apache::grades::getclasslist(\@chosen_sections,);
    &Apache::grades::reset_perm();
    
    my $result;
    if (!(grep(/^all$/,@chosen_sections))) {
	$result.='<p> Showing only sections <tt>'.join(', ',@chosen_sections).
	    '</tt>.</p> '."\n";
    }
    $result.='<table border="1">';

    foreach my $student (sort {lc($$fullname{$a}) cmp lc($$fullname{$b}) } keys %$fullname) {
	my ($uname,$udom) = split(/:/,$student);
	
	my $cnum=$env{'course.'.$cid.'.num'};
	my $cdom=$env{'course.'.$cid.'.domain'};
	my %status = &get_student_status($symb,$cdom,$cnum,$udom,$uname,
					 'Task');
	my $queue = 'none';
	my $cannot_grade;
	if ($status{'reviewqueue'} =~ /^(in_progress|enqueue)$/) {
	    $queue = 'reviewqueue';
	    if ($status{'reviewqueue'} eq 'in_progress') {
		$cannot_grade=1;
	    }
	} elsif ($status{'gradingqueue'} =~ /^(in_progress|enqueue)$/) {
	    $queue = 'gradingqueue';
	    if ($status{'gradingqueue'} eq 'in_progress') {
		$cannot_grade=1;
	    }
	}
	my $todo = 
	    &Apache::lonnet::escape(&encode_queue_key($symb,$udom,$uname));
	if ($cannot_grade) {
	    $result.='<tr><td>&nbsp;</td><td>'.$fullname->{$student}.
		'</td><td>';
	} else {
	    my $seclist;
	    foreach my $sec (@chosen_sections) {
		$seclist.='<input type="hidden" name="chosensections" 
                               value="'.$sec.'" />';
	    }
	    $result.=<<RESULT;
<tr>
  <td>
    <form method="POST">
      <input type="hidden" name="gradingkey" value="$todo" />
      <input type="hidden" name="queue" value="$queue" />
      <input type="hidden" name="webgrade" value="no" />
      <input type="hidden" name="regrade" value="yes" />
      <input type="submit" name="submit" value="Regrade" />
      $seclist
    </form>
  <td>$fullname->{$student}</td>
  <td>
RESULT
        }
        if ($status{'status'} eq 'pass') {
	    $result .= '<font color="green">'.&mt('Passed').'</font>';
	} elsif ($status{'status'} eq 'fail') {
	    $result .= '<font color="red">'.&mt('Failed').'</font>';
	} elsif ($status{'status'} eq 'review') {
	    $result .= '<font color="blue">'.&mt('Under Review').'</font>';
	} elsif ($status{'status'} eq 'ungraded') {
	    $result .= &mt('Ungraded');
	} elsif ($status{'status'} ne '') {
	    $result .= '<font color="orange">'.&mt('Unknown Status').'</font>';
	} else {
	    $result.="&nbsp;";
	}
	if ($status{'version'}) {
	    $result .= ' '.&mt('Version').' '.$status{'version'};
	}
	$result.= '</td><td>';
	if ($status{'reviewqueue'} eq 'enqueued') {
	    $result .= &mt('Awaiting Review');
	} elsif ($status{'reviewqueue'} eq 'locked') {
	    $result .= &mt('Under Review');
	} elsif ($status{'reviewqueue'} eq 'in_progress') {
	    $result .= &mt('Still being worked on.');
	} elsif ($status{'gradingqueue'} eq 'enqueued') {
	    $result .= &mt('Awaiting Grading');
	} elsif ($status{'gradingqueue'} eq 'locked') {
	    $result .= &mt('Being Graded');
	} elsif ($status{'gradingqueue'} eq 'in_progress') {
	    $result .= &mt('Still being worked on.');
	} else {
	    $result.="&nbsp;";
	}
	$result.= '</td></tr>';
    }
    $result.='</table>';
    return $result;
}

sub get_student_status {
    my ($symb,$cdom,$cnum,$udom,$uname,$type)=@_;

    my %status;

    if ($type eq 'Task') {
	my %record = &Apache::lonnet::restore($symb,$env{'request.course.id'},
					  $udom,$uname);
	$status{'status'}=$record{'resource.0.status'};
	$status{'version'}=$record{'resource.0.version'};
	$status{'grader'}=$record{'resource.0.regrader'};
    }
    $status{'reviewqueue'}=
	&check_queue_for_key($cdom,$cnum,'reviewqueue',
			     &encode_queue_key($symb,$udom,$uname));
    $status{'gradingqueue'}=
	&check_queue_for_key($cdom,$cnum,'gradingqueue',
			     &encode_queue_key($symb,$udom,$uname));
    return %status;
}

sub start_ClosingParagraph {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
    my $result;
    if ($target eq 'web') {
    } elsif ($target eq 'webgrade') {
	&Apache::lonxml::startredirection();
    }
    return $result;
}

sub end_ClosingParagraph {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
    my $result;
    if ($target eq 'web') {
    } elsif ($target eq 'webgrade') {
	&Apache::lonxml::endredirection();
    }
    return $result;
}

sub get_id {
    my ($parstack,$safeeval)=@_;
    my $id=&Apache::lonxml::get_param('id',$parstack,$safeeval);
    if (!$id) { $id=$Apache::lonxml::curdepth; }
    return $id;
}

my %dimension;
sub start_Dimension {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
    undef(%dimension);
    my $dim_id=&get_id($parstack,$safeeval);
    $Apache::bridgetask::dimension=$dim_id;
    push(@Apache::bridgetask::dimensionlist,$dim_id);
    undef(@Apache::bridgetask::instance);
    $Apache::bridgetask::dimensionmandatory{$dim_id}=
	&Apache::lonxml::get_param('Mandatory',$parstack,$safeeval);
    &Apache::lonxml::startredirection();
    return &internal_location($dim_id);
}

sub get_instance {
    my ($dim)=@_;
    my $rand_alg=&Apache::lonnet::get_rand_alg();
    if (!$rand_alg || $rand_alg eq '32bit' || $rand_alg eq '64bit' ||
	$rand_alg eq '64bit2' || $rand_alg eq '64bit3' ||
	$rand_alg eq '64bit4' ) {
	&Apache::response::pushrandomnumber();
	my @order=&Math::Random::random_permutation(@{$dimension{'instances'}});
	my $num=@order;
	my $version=&get_version();
	my $which=($version-1)%$num;
	return $order[$which];
    } else {
	my ($version,$previous) = &get_version();
	my $instance = 
	    $Apache::lonhomework::history{"resource.$version.0.$dim.instance"};
	if (defined($instance)) { return $instance; }

	&Apache::response::pushrandomnumber();
	my @instances = @{$dimension{'instances'}};
	# remove disabled instances
	for (my $i=0; $i < $#instances; $i++) {
	    if ($dimension{$instances[$i].'.disabled'}) {
		splice(@instances,$i,1);
		$i--;
	    }
	}
	@instances = &Math::Random::random_permutation(@instances);
	$instance  = $instances[($version-1)%scalar(@instances)];
	$Apache::lonhomework::results{"resource.$version.0.$dim.instance"} = 
	    $instance;
	$Apache::lonhomework::results{'INTERNAL_store'} = 1; 
	&Apache::response::poprandomnumber();
	return $instance;
    }
}

{
    my $last_link;
    sub end_Dimension {
	my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
	my $result=&Apache::lonxml::endredirection();
	my $dim=&get_id($parstack,$safeeval);
	my $instance=&get_instance($dim);
	my $version=&get_version();
	if ($target eq 'web') {
	    @Apache::scripttag::parser_env = @_;
	    $result.=&Apache::scripttag::xmlparse($dimension{'intro'});
	    my @instances = $instance;
	    if (&Apache::response::showallfoils()) {
		@instances = @{$dimension{'instances'}};
	    }
	    foreach my $instance (@instances) {
		@Apache::scripttag::parser_env = @_;
		$result.=&Apache::scripttag::xmlparse($dimension{$instance.'.text'});
		if ($Apache::lonhomework::history{"resource.$version.0.status"} eq 'pass' ||
		    $Apache::lonhomework::history{"resource.$version.0.status"} eq 'fail') {

		    my $dim_status=$Apache::lonhomework::history{"resource.$version.0.$dim.status"};
		    my $mandatory='Mandatory';
		    if ($Apache::bridgetask::dimensionmandatory{$dim} eq 'N') {
			$mandatory='Optional';
		    }
		    my $dim_info="<div class='$dim_status'>\n";
		    if ($dim_status eq 'pass') {
			$dim_info.='<h3>Question : you passed this '.$mandatory.' question</h3>';
		    }
		    if ($dim_status eq 'fail') {
			$dim_info.='<h3>Question : you did not pass this '.$mandatory.' question</h3>';
		    }
		    my $man_count=0;
		    my $man_passed=0;
		    my $opt_count=0;
		    my $opt_passed=0;
		    foreach my $id (@{$dimension{$instance.'.criterias'}}) {
			if ($dimension{$instance.'.criteria.'.$id.'.mandatory'} 
			    eq 'N') {
			    $opt_count++;
			    if ($Apache::lonhomework::history{"resource.$version.0.$dim.$instance.$id.status"} eq 'pass') {
				$opt_passed++;
			    }
			} else {
			    $man_count++;
			    if ($Apache::lonhomework::history{"resource.$version.0.$dim.$instance.$id.status"} eq 'pass') {
				$man_passed++;
			    }
			}
		    }
		    if ($man_passed eq $man_count) { $man_passed='all'; }
		    my $opt_req=$dimension{$instance.'.optionalrequired'};
		    if ($opt_req !~ /\S/) { $opt_req='0'; }
		    $dim_info.="\n<p>".&mt('You passed [_1] of the [_2] mandatory components and [_3] of the [_4] optional components, of which you were required to pass [_5].',$man_passed,$man_count,$opt_passed,$opt_count,$opt_req)."</p>\n</div>";

		    my $internal_location=&internal_location($dim);
		    $result=~s/\Q$internal_location\E/$dim_info/;

		    foreach my $id (@{$dimension{$instance.'.criterias'}}) {
			my $status=$Apache::lonhomework::history{"resource.$version.0.$dim.$instance.$id.status"};
			my $comment=$Apache::lonhomework::history{"resource.$version.0.$dim.$instance.$id.comment"};
			my $mandatory=($dimension{$instance.'.criteria.'.$id.'.mandatory'} ne 'N');
			if ($mandatory) {
			    $mandatory='Mandatory';
			} else {
			    $mandatory='Optional';
			}
			if ($status eq 'fail') {
			} elsif ($status eq 'pass') {
			} else {
			    &Apache::lonxml::error("Student viewing a graded bridgetask was shown a status of $status");
			}
			my $status_display=$status;
			$status_display=~s/^([a-z])/uc($1)/e;
			@Apache::scripttag::parser_env = @_;
			$result.='<div class="'.$status.'"><h4>'.$mandatory.
			    ' Criteria</h4><p>';
			@Apache::scripttag::parser_env = @_;
			$result.=&Apache::scripttag::xmlparse($dimension{$instance.'.criteria.'.$id});
			$result.='</p><p class="grade">'.$status_display.'</p>';
			if ($Apache::lonhomework::history{"resource.$version.0.$dim.$instance.$id.comment"}) {
			    $result.='<p class="comment">'.$Apache::lonhomework::history{"resource.$version.0.$dim.$instance.$id.comment"}.'</p>';
			}
			$result.='</div>';
		    }
		}
	    }
	} elsif ($target eq 'webgrade') {
	    # in case of any side effects that we need
	    @Apache::scripttag::parser_env = @_;
	    &Apache::scripttag::xmlparse($dimension{'intro'});
	    @Apache::scripttag::parser_env = @_;
	    &Apache::scripttag::xmlparse($dimension{$instance.'.text'});
	    foreach my $id (@{$dimension{$instance.'.criterias'}}) {
		my $link='criteria_'.$instance.'_'.$id;
		my $status=$Apache::lonhomework::history{"resource.$version.0.$dim.$instance.$id.status"};
		$result.='<tr><td width="100%" valign="top">'.
		    '<a name="'.$link.'" />'.
		    '<a name="next_'.$last_link.'" />'.
		    '<br /><textarea enabled="false" style="width:100%" rows="8" width="25" wrap="hard">';
		@Apache::scripttag::parser_env = @_;
		$result.=&Apache::scripttag::xmlparse($dimension{$instance.'.criteria.'.$id});
		$result.='</textarea>'.
		    #$dimension{$instance.'.criteria.'.$id}.
		    '</td>'.
		    '<td><nobr>Additional Comment for Student</nobr> <br />'.
		    '<textarea style="width:100%" rows="8" width="25" wrap="hard" name="HWVAL_comment_'.$link.'">'.&HTML::Entities::encode($Apache::lonhomework::history{"resource.$version.0.$dim.$instance.$id.comment"}).'</textarea>'.
		    '</td>'.
		    '<td>'.
		    '<nobr><label><input type="radio" name="HWVAL_'.$link.'" value="ungraded" '.($status eq 'ungraded' || !$status ? 'checked="checked"':'').' />'.&mt('Ungraded').'</label></nobr><br />'.
		    '<label><input type="radio" name="HWVAL_'.$link.'" value="pass" '.($status eq 'pass' ? 'checked="checked"':'').' />'.&mt('Pass').'</label><br />'.
		    '<label><input type="radio" name="HWVAL_'.$link.'" value="fail" '.($status eq 'fail' ? 'checked="checked"':'').' />'.&mt('Fail').'</label><br />'.
		    '<label><input type="radio" name="HWVAL_'.$link.'" value="review" '.($status eq 'review' ? 'checked="checked"':'').' />'.&mt('Review').'</label><br />'.
		    '</td>'.
		    '<td>'.
		    '<a href="#'.$last_link.'">Prev</a><br />'.
		    '<a href="#next_'.$link.'">Next</a><br /><br /><br />'.
		    '</td></tr>';
		$last_link=$link;
	    }
	} elsif ($target eq 'grade' && $env{'form.webgrade'}) {
	    my $optional_passed=0;
	    my $mandatory_failed=0;
	    my $ungraded=0;
	    my $review=0;
	    foreach my $id (@{$dimension{$instance.'.criterias'}}) {
		my $status=$Apache::lonhomework::results{"resource.$version.0.$dim.$instance.$id.status"}=$env{'form.HWVAL_criteria_'.$instance.'_'.$id};
		$Apache::lonhomework::results{"resource.$version.0.$dim.$instance.$id.comment"}=$env{'form.HWVAL_comment_criteria_'.$instance.'_'.$id};
		my $mandatory=($dimension{$instance.'.criteria.'.$id.'.mandatory'} ne 'N');
		if ($status eq 'pass') {
		    if (!$mandatory) { $optional_passed++; }
		} elsif ($status eq 'fail') {
		    if ($mandatory) { $mandatory_failed++; }
		} elsif ($status eq 'review') {
		    $review++;
		} elsif ($status eq 'ungraded') {
		    $ungraded++;
		} else {
		    $ungraded++;
		}
	    }
	    if ($optional_passed < $dimension{$instance.'.optionalrequired'}) {
		$mandatory_failed++;
	    }
	    &Apache::lonxml::debug("all instance ".join(':',@{$dimension{$instance.'.criterias'}})." results -> m_f $mandatory_failed o_p $optional_passed u $ungraded r $review");
	    if ($review) {
		$Apache::lonhomework::results{"resource.$version.0.$dim.status"}=
		    'review';
	    } elsif ($ungraded) {
		$Apache::lonhomework::results{"resource.$version.0.$dim.status"}=
		    'ungraded';
	    } elsif ($mandatory_failed) {
		$Apache::lonhomework::results{"resource.$version.0.$dim.status"}=
		    'fail';
	    } else {
		$Apache::lonhomework::results{"resource.$version.0.$dim.status"}=
		    'pass';
	    }
	} else {
	    # any other targets no output
	    undef($result);
	}
	return $result;
    }
}

sub start_IntroParagraph {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
    my $result;
    if ($target eq 'web' || $target eq 'webgrade') {
	if ($tagstack->[-2] eq 'Dimension') {
	    $dimension{'intro'}=&Apache::lonxml::get_all_text('/introparagraph',$parser,$style);
	
	} elsif ($target eq 'webgrade') {
	    &Apache::lonxml::get_all_text('/introparagraph',$parser,$style);
	}
	
    }
    return $result;
}

sub end_IntroParagraph {
}

sub start_Instance {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
    my $id=&get_id($parstack,$safeeval);
    push(@{$dimension{'instances'}},$id);
    push(@Apache::bridgetask::instance,$id);
    push(@Apache::bridgetask::instancelist,$id);
    $dimension{$id.'.optionalrequired'}=
	&Apache::lonxml::get_param('OptionalRequired',$parstack,$safeeval);
    my $disabled = &Apache::lonxml::get_param('Disabled',$parstack,$safeeval);
    if (lc($disabled) eq 'yes') {
	$dimension{$id.'.disabled'}='1';
    }
    return '';
}

sub end_Instance {
}

sub start_InstanceText {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
    my $instance_id=$Apache::bridgetask::instance[-1];
    my $text=&Apache::lonxml::get_all_text('/instancetext',$parser,$style);
    if ($target eq 'web' || $target eq 'webgrade') {
	$dimension{$instance_id.'.text'}=$text;
    }
    return '';
}

sub end_InstanceText {
    return '';
}

sub start_Criteria {
    my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
    my $instance_id=$Apache::bridgetask::instance[-1];
    my $criteria=&Apache::lonxml::get_all_text('/criteria',$parser,$style);
    if ($target eq 'web' || $target eq 'webgrade' || $target eq 'grade') {
	my $id=&get_id($parstack,$safeeval);
	$dimension{$instance_id.'.criteria.'.$id}=$criteria;
	$dimension{$instance_id.'.criteria.'.$id.'.mandatory'}=
	    &Apache::lonxml::get_param('Mandatory',$parstack,$safeeval);
	push(@{$dimension{$instance_id.'.criterias'}},$id);
    }
    return '';
}

sub end_Criteria {
}

sub proctor_validation_screen {
    my ($slot) = @_;
    my (undef,undef,$domain,$user) = &Apache::lonxml::whichuser();
    my $url=&Apache::lonnet::studentphoto($domain,$user,'jpg');
    my $name=&Apache::loncommon::plainname($user,$domain);
    
    my $msg;
    if ($env{'form.proctorpassword'}) {
	$msg='<p><font color="red">'.&mt("Failed to authenticate the proctor.")
	    .'</font></p>';
    }
    if (!$env{'form.proctordomain'}) { $env{'form.proctordomain'}=$domain; }
    my $result= (<<ENDCHECKOUT);
<h2>Proctor Validation</h2>
    <p>Your room's proctor needs to validate your access to this resource.</p>
    $msg
<form name="checkout" method="POST" action="$env{'request.uri'}">
<input type="hidden" name="validate" value="yes" />
<input type="hidden" name="submitted" value="yes" />
<table>
  <tr><td>Proctor's Username:</td><td><input type="string" name="proctorname" value="$env{'form.proctorname'}" /></td></tr>
  <tr><td>Password:</td><td><input type="password" name="proctorpassword" value="" /></td></tr>
  <tr><td>Proctor's Domain:</td><td><input type="string" name="proctordomain" value="$env{'form.proctordomain'}" /></td></tr>
</table>
<input type="submit" name="checkoutbutton" value="Validate"  /><br />
<table border="1">
  <tr><td>
    <table>
      <tr><td colspan="2">Student who should be logged in is:</td></tr>
      <tr><td>Name:</td><td>$name</td></tr>
      <tr><td>Student ID:</td><td>$env{'environment.id'}</td></tr>
      <tr><td>Usename</td><td>$user\@$domain</td></tr>
      <tr><td colspan="2"><img src="$url" /></td></tr>
    </table>
  </tr></td>
</table>
</form>
ENDCHECKOUT
    return $result;
}

1;
__END__
