# The LearningOnline Network
# RSS Feeder
#
# $Id: lonrss.pm,v 1.7 2005/11/22 16:16:53 albertel Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#

package Apache::lonrss;

use strict;
use Apache::Constants qw(:common);
use Apache::loncommon;
use Apache::lonnet;
use Apache::lontexconvert;
use Apache::lonlocal;
use Apache::lonhtmlcommon;

sub filterfeedname {
    my $filename=shift;
    $filename=~s/(\_rss\.html|\.rss)$//;
    $filename=~s/\W//g;
    return $filename;
}

sub feedname {
    return 'nohist_'.&filterfeedname(shift).'_rssfeed';
}

sub displayfeedname {
    my ($rawname,$uname,$udom)=@_;
    my $filterfilename=&filterfeedname($rawname);
# do we have a stored name?
    my %stored=&Apache::lonnet::get('nohist_all_rss_feeds',[$filterfilename],$udom,$uname);
    if ($stored{$filterfilename}) { return $stored{$filterfilename}; }
# no, construct a name
    my $name=$filterfilename; 
    if ($name=~/^CourseBlog/) {
        $name=&mt('Course Blog');
	if ($env{'course.'.$env{'request.course.id'}.'.description'}) {
	    $name.=' '.$env{'course.'.$env{'request.course.id'}.'.description'};
	}
    } else {
	$name=~s/\_/ /g;
    }
    return $name;
}

sub renamefeed {
    my ($rawname,$uname,$udom,$newname)=@_;
    return &Apache::lonnet::put('nohist_all_rss_feeds',
				{ &filterfeedname($rawname) => $newname },
				$udom,$uname);
}

sub advertisefeeds {
    return;
    my ($uname,$udom,$edit)=@_;
    my $feeds='';
    my %feednames=&Apache::lonnet::dump('nohist_all_rss_feeds',$udom,$uname);
    my $mode='public';
    if ($edit) {
	$mode='adm';
    }
    foreach my $feed (sort(keys(%feednames))) {
	if ($feed!~/^error\:/) {
	    my $feedurl='feed://'.$ENV{'HTTP_HOST'}.'/public/'.$udom.'/'.$uname.'/'.$feed.'.rss';
	    my $htmlurl='http://'.$ENV{'HTTP_HOST'}.'/'.$mode.'/'.$udom.'/'.$uname.'/'.$feed.'_rss.html';
	    $feeds.='<li>'.$feednames{$feed}.
		'<br />'.($edit?&mt('Edit'):'HTML').': <a href="'.$htmlurl.'"><tt>'.$htmlurl.'</tt></a>'.
		($edit?'':'<br />RSS: <a href="'.$feedurl.'"><tt>'.$feedurl.'</tt></a>').'</li>';
	}
    }
    if ($feeds) {
	return '<h4>'.&mt('Available RSS Feeds and Blogs').'</h4><ul>'.$feeds.'</ul>';
    } else {
        return '';
    }
}

{
    my $feedcounter;
    sub addentry {
	$feedcounter++;
	my $id=time.'00000'.$$.'00000'.$feedcounter;
	return &editentry($id,@_);
    }
}

sub editentry {
    my ($id,$uname,$udom,$filename,$title,$description,$url,$status,$encurl,$enclength,$enctype)=@_;
    my $feedname=&feedname($filename);
    &Apache::lonnet::put('nohist_all_rss_feeds',
			 { &filterfeedname($filename) => &displayfeedname($filename,$uname,$udom) },
			 $udom,$uname);
    return &Apache::lonnet::put($feedname,{
	$id.'_title' => $title,
	$id.'_description' => $description,
	$id.'_link' => $url,
	$id.'_enclosureurl' => $encurl,
	$id.'_enclosurelength' => $enclength,
	$id.'_enclosuretype' => $enctype,
	$id.'_status' => $status},$udom,$uname);
}

sub changestatus {
    my ($id,$uname,$udom,$filename,$status)=@_;
    my $feedname=&feedname($filename);
    if ($status eq 'deleted') {
	return &Apache::lonnet::del($feedname,[$id.'_title',
					       $id.'_description',
					       $id.'_link',
					       $id.'_enclosureurl',
					       $id.'_enclosurelength',
					       $id.'_enclosuretype',
					       $id.'_status'],$udom,$uname);
    } else {
	return &Apache::lonnet::put($feedname,{$id.'_status' => $status},$udom,$uname);
    }
}

sub handler {
    my $r = shift;

    my $edit=0;
    my $html=0;
    my (undef,$mode,$udom,$uname,$filename)=split(/\//,$r->uri);
    if (($mode eq 'adm') && ($env{'user.name'} eq $uname) && ($env{'user.domain'} eq $udom)) {
	$edit=1;
	$html=1;
    }
    if ($filename=~/\.html$/) {
	$html=1;
    }
    if ($html) {
	&Apache::loncommon::content_type($r,'text/html');
    } else {
# Workaround Mozilla/Firefox
#	&Apache::loncommon::content_type($r,'application/rss+xml');
	&Apache::loncommon::content_type($r,'text/xml');
    }
    $r->send_http_header;
    return OK if $r->header_only;

    my $filterfeedname=&filterfeedname($filename);
    my $feedname=&feedname($filename);
    my $displayfeedname=&displayfeedname($filename,$uname,$udom);
    if ($html) {
	$r->print(&Apache::lonxml::xmlbegin().&Apache::loncommon::bodytag
                  ($displayfeedname,'','','',$udom,
                   $env{'form.register'}).<<ENDSCRIPT);
<script>
    function changed(tform,id) {
        tform.elements[id+"_modified"].checked=true;
    }
</script>
ENDSCRIPT
    } else {
	$r->print("<rss version='2.0' xmlns:dc='http://purl.org/dc/elements/1.1'>\n<channel>".
		  "\n<link>http://".$ENV{'HTTP_HOST'}.'/public/'.$udom.'/'.$uname.'/'.
		  $filterfeedname.'_rss.html</link>'.
		  "\n<description>".
		  &mt('An RSS Feed provided by the LON-CAPA Learning Content Management System').
		  '</description>');
    }
# Is this user for real?
    my $homeserver=&Apache::lonnet::homeserver($uname,$udom);
    if ($html) {
	$r->print(&advertisefeeds($uname,$udom,$edit));
    } 
    if ($homeserver eq 'no_host') {
	$r->print(($html?'<h3>':'<title>').&mt('No feed available').($html?'</h3>':'</title>'));
    } else {
# Course or user?
	my $name='';
	if ($uname=~/^\d/) {
	    my %cenv=&Apache::lonnet::dump('environment',$udom,$uname);
	    $name=$cenv{'description'};
	} else {
	    $name=&Apache::loncommon::nickname($uname,$udom);
	}
        $r->print("\n".
		  ($html?'<h3>':'<title>').
		  &mt('LON-CAPA Feed "[_1]" for [_2]',$displayfeedname,$name).
		  ($html?'</h3>'.($edit?'<form method="post"><br />'.
				  &mt('Name of blog/journal').
				  ': <input type="text" size="50" name="newblogname" value="'.
				  $displayfeedname.'" />':'').'<ul>':'</title>'));
# Render private items?
        my $viewpubliconly=1;
        if (($env{'user.name'} eq $uname) && ($env{'user.domain'} eq $udom)) {
	    $viewpubliconly=0;
	}
# Get feed items
        my %newsfeed=&Apache::lonnet::dump($feedname,$udom,$uname);
	foreach my $entry (sort(keys(%newsfeed))) {
	    if ($entry=~/^(\d+)\_status$/) {
		my $id=$1;
		if ($edit) {
		    my %lt=&Apache::lonlocal::texthash('public' => 'public',
						       'private' => 'private',
						       'hidden' => 'hidden',
						       'delete' => 'delete',
						       'store' => 'Store changes');
		    my %status=();
		    $status{$newsfeed{$id.'_status'}}='checked="checked"';
		    $r->print(<<ENDEDIT);
<li>
<label><input name='$id\_modified' type='checkbox' /> $lt{'store'}</label>
&nbsp;&nbsp;
<label><input name='$id\_status' type="radio" value="public" $status{'public'} onClick="changed(this.form,'$id');" /> $lt{'public'}</label>
&nbsp;&nbsp;
<label><input name='$id\_status' type="radio" value="private" $status{'private'} onClick="changed(this.form,'$id');" /> $lt{'private'}</label>
&nbsp;&nbsp;
<label><input name='$id\_status' type="radio" value="hidden" $status{'hidden'} onClick="changed(this.form,'$id');" /> $lt{'hidden'}</label>
&nbsp;&nbsp;
<label><input name='$id\_status' type="radio" value="delete" onClick="changed(this.form,'$id');" /> $lt{'delete'}</label>
<br />
<input name='$id\_title' type='text' size='80' value='$newsfeed{$id.'_title'}' onChange="changed(this.form,'$id');" /><br />
<textarea name='$id\_description' rows="6" cols="80" onChange="changed(this.form,'$id');">$newsfeed{$id.'_description'}</textarea><br />
<input name='$id\_link' type='text' size='80' value='$newsfeed{$id.'_link'}' onChange="changed(this.form,'$id');" />
<hr /></li>
ENDEDIT
		} else {
		    if (($newsfeed{$id.'_status'} ne 'public') && ($viewpubliconly)) { next; }
		    if ($newsfeed{$id.'_status'} eq 'hidden') { next; }
		    $r->print("\n".($html?"\n<li><b>":"<item>\n<title>").$newsfeed{$id.'_title'}.
			      ($html?"</b><br />\n":"</title>\n<description>").
			      $newsfeed{$id.'_description'}.
			      ($html?"<br />\n<a href='":"</description>\n<link>").
			      "http://".$ENV{'HTTP_HOST'}.
			      $newsfeed{$id.'_link'}.
			      ($html?("'>".&mt('Read more')."</a><br />\n"):"</link>\n"));
		    if ($newsfeed{$id.'_enclosureurl'}) {
			$r->print(($html?"<a href='":"\n<enclosure url='").
				  $newsfeed{$id.'_enclosureurl'}."' length='".$newsfeed{$id.'_enclosurelength'}.
				  "' type='".$newsfeed{$id.'_enclosuretype'}.($html?"'>".&mt('Enclosure')."</a>":"' />"));
		    }
		    if ($html) {
			$r->print("\n<hr /></li>\n");
		    } else {
			$r->print("\n<guid isPermaLink='false'>".$id.$filterfeedname.'_'.$udom.'_'.$uname."</guid></item>\n");
		    }
		}
	    }
	}
    }
    $r->print("\n".($html?'</ul>'.($edit?'<input type="submit" value="'.&mt('Store Marked Changes').'" /></form>':'').'</body></html>':'</channel></rss>'."\n"));
    return OK;
} 
1;
__END__
